@@ -1,8 +1,22 @@
+#include <libfilezilla/libfilezilla.hpp>
+#ifdef FZ_WINDOWS
+  #include <libfilezilla/private/windows.hpp>
+  #include <winsock2.h>
+  #include <ws2tcpip.h>
+#else
+  #include <sys/socket.h>
+  #include <netdb.h>
+#endif
 #include <filezilla.h>
 #include "engineprivate.h"
 #include "proxy.h"
 #include "ControlSocket.h"
-#include <wx/sckaddr.h>
+
+#include <libfilezilla/iputils.hpp>
+
+#include <algorithm>
+
+#include <string.h>
 
 enum handshake_state
 {
@@ -17,89 +31,56 @@ enum handshake_state
 	socks4_handshake
 };
 
-CProxySocket::CProxySocket(CEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
-	: CEventHandler(pOwner->event_loop_)
+CProxySocket::CProxySocket(fz::event_handler* pEvtHandler, fz::socket* pSocket, CControlSocket* pOwner)
+	: fz::event_handler(pOwner->event_loop_)
 	, CBackend(pEvtHandler)
-	, m_pSocket(pSocket)
+	, socket_(pSocket)
 	, m_pOwner(pOwner)
 {
-	m_pSocket->SetEventHandler(this);
+	socket_->set_event_handler(this);
 }
 
 CProxySocket::~CProxySocket()
 {
-	RemoveHandler();
+	remove_handler();
 
-	if (m_pSocket)
-		m_pSocket->SetEventHandler(0);
+	if (socket_) {
+		socket_->set_event_handler(0);
+	}
 	delete [] m_pSendBuffer;
 	delete [] m_pRecvBuffer;
 }
 
-namespace {
-wxString base64encode(const wxString& str)
-{
-	// Code shamelessly taken from wxWidgets and adopted to encode UTF-8 strings.
-	// wxWidget's http class encodes string from arbitrary encoding into base64,
-	// could as well encode /dev/random
-	static char const*const base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
-	wxString buf;
-
-	const wxWX2MBbuf utf8 = str.mb_str(wxConvUTF8);
-	const char* from = utf8;
-
-	size_t len = strlen(from);
-	while (len >= 3) { // encode full blocks first
-		buf << wxString::Format(wxT("%c%c"), base64[(from[0] >> 2) & 0x3f], base64[((from[0] << 4) & 0x30) | ((from[1] >> 4) & 0xf)]);
-		buf << wxString::Format(wxT("%c%c"), base64[((from[1] << 2) & 0x3c) | ((from[2] >> 6) & 0x3)], base64[from[2] & 0x3f]);
-		from += 3;
-		len -= 3;
-	}
-	if (len > 0) { // pad the remaining characters
-		buf << wxString::Format(wxT("%c"), base64[(from[0] >> 2) & 0x3f]);
-		if (len == 1) {
-			buf << wxString::Format(wxT("%c="), base64[(from[0] << 4) & 0x30]);
-		}
-		else {
-			buf << wxString::Format(wxT("%c%c"), base64[((from[0] << 4) & 0x30) | ((from[1] >> 4) & 0xf)], base64[(from[1] << 2) & 0x3c]);
-		}
-		buf << wxString::Format(wxT("="));
-	}
-
-	return buf;
-}
-}
-
-wxString CProxySocket::Name(ProxyType t)
+std::wstring CProxySocket::Name(ProxyType t)
 {
 	switch (t) {
 	case HTTP:
-		return _T("HTTP");
+		return L"HTTP";
 	case SOCKS4:
-		return _T("SOCKS4");
+		return L"SOCKS4";
 	case SOCKS5:
-		return _T("SOCKS5");
+		return L"SOCKS5";
 	default:
 		return _("unknown");
 	}
 }
 
-int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass)
+int CProxySocket::Handshake(CProxySocket::ProxyType type, std::wstring const& host, unsigned int port, std::wstring const& user, std::wstring const& pass)
 {
-	if (type == CProxySocket::unknown || host.empty() || port < 1 || port > 65535)
+	if (type == CProxySocket::unknown || host.empty() || port < 1 || port > 65535) {
 		return EINVAL;
+	}
 
-	if (m_proxyState != noconn)
+	if (m_proxyState != noconn) {
 		return EALREADY;
+	}
 
-	const wxWX2MBbuf host_raw = host.mb_str(wxConvUTF8);
-
-	if (type != HTTP && type != SOCKS5 && type != SOCKS4)
+	if (type != HTTP && type != SOCKS5 && type != SOCKS4) {
 		return EPROTONOSUPPORT;
+	}
 
-	m_user = user;
-	m_pass = pass;
+	m_user = fz::to_utf8(user);
+	m_pass = fz::to_utf8(pass);
 	m_host = host;
 	m_port = port;
 	m_proxyType = type;
@@ -109,46 +90,55 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host,
 	if (type == HTTP) {
 		m_handshakeState = http_wait;
 
-		wxWX2MBbuf challenge{};
-		int challenge_len{};
+		std::string auth;
 		if (!user.empty()) {
-			challenge = base64encode(user + _T(":") + pass).mb_str(wxConvUTF8);
-			challenge_len = strlen(challenge);
+			auth = "Proxy-Authorization: Basic ";
+			auth += fz::base64_encode(m_user + ":" + m_pass);
+			auth += "\r\n";
 		}
 
 		// Bit oversized, but be on the safe side
-		m_pSendBuffer = new char[70 + strlen(host_raw) * 2 + 2*5 + challenge_len + 23];
+		std::string host_raw = fz::to_utf8(host);
+		m_pSendBuffer = new char[70 + host_raw.size() * 2 + 2*5 + auth.size() + 23];
 
-		if (!challenge || !challenge_len) {
-			m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nUser-Agent: FileZilla\r\n\r\n",
-				(const char*)host_raw, port,
-				(const char*)host_raw, port);
-		}
-		else {
-			m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nProxy-Authorization: Basic %s\r\nUser-Agent: FileZilla\r\n\r\n",
-				(const char*)host_raw, port,
-				(const char*)host_raw, port,
-				(const char*)challenge);
-		}
+		m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\n%sUser-Agent: %s\r\n\r\n",
+			host_raw.c_str(), port,
+			host_raw.c_str(), port,
+			auth.c_str(),
+			fz::replaced_substrings(PACKAGE_STRING, " ", "/").c_str());
 
 		m_pRecvBuffer = new char[4096];
 		m_recvBufferLen = 4096;
 		m_recvBufferPos = 0;
 	}
 	else if (type == SOCKS4) {
-		wxString ip = m_host;
-		if (!GetIPV6LongForm(m_host).empty()) {
+		std::string ip;
+		auto const addressType = fz::get_address_type(m_host);
+		if (addressType == fz::address_type::ipv6) {
 			m_pOwner->LogMessage(MessageType::Error, _("IPv6 addresses are not supported with SOCKS4 proxy"));
 			return EINVAL;
 		}
+		else if (addressType == fz::address_type::ipv4) {
+			ip = fz::to_string(m_host);
+		}
+		else {
+			addrinfo hints{};
+			hints.ai_family = AF_INET;
+			hints.ai_socktype = SOCK_STREAM;
 
-		if (!IsIpAddress(m_host)) {
-			wxIPV4address address;
-			if (!address.Hostname(host)) {
+			addrinfo * result{};
+			int res = getaddrinfo(fz::to_string(m_host).c_str(), 0, &hints, &result);
+			if (!res && result) {
+				if (result->ai_family == AF_INET) {
+					ip = fz::socket::address_to_string(result->ai_addr, result->ai_addrlen, false);
+				}
+				freeaddrinfo(result);
+			}
+
+			if (ip.empty()) {
 				m_pOwner->LogMessage(MessageType::Error, _("Cannot resolve hostname to IPv4 address for use with SOCKS4 proxy."));
 				return EINVAL;
 			}
-			ip = address.IPAddress();
 		}
 
 		m_pOwner->LogMessage(MessageType::Status, _("SOCKS4 proxy will connect to: %s"), ip);
@@ -161,10 +151,10 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host,
 		unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
 		int i = 0;
 		memset(buf, 0, 4);
-		for (const wxChar* p = ip.c_str(); *p && i < 4; p++) {
-			const wxChar& c = *p;
+		for (auto p = ip.c_str(); *p && i < 4; ++p) {
+			auto const& c = *p;
 			if (c == '.') {
-				i++;
+				++i;
 				continue;
 			}
 			buf[i] *= 10;
@@ -178,6 +168,11 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host,
 		m_handshakeState = socks4_handshake;
 	}
 	else {
+		if (m_user.size() > 255 || m_pass.size() > 255) {
+			m_pOwner->LogMessage(MessageType::Status, _("SOCKS5 does not support usernames or passwords longer than 255 characters."));
+			return EINVAL;
+		}
+
 		m_pSendBuffer = new char[4];
 		m_pSendBuffer[0] = 5; // Protocol version
 		if (!user.empty()) {
@@ -202,62 +197,66 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host,
 	return EINPROGRESS;
 }
 
-void CProxySocket::operator()(CEventBase const& ev)
+void CProxySocket::operator()(fz::event_base const& ev)
 {
-	Dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+	fz::dispatch<fz::socket_event, fz::hostaddress_event>(ev, this,
 		&CProxySocket::OnSocketEvent,
 		&CProxySocket::OnHostAddress);
 }
 
-void CProxySocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
+void CProxySocket::OnSocketEvent(socket_event_source*, fz::socket_event_flag t, int error)
 {
 	switch (t) {
-	case SocketEventType::connection_next:
-		if (error)
-			m_pOwner->LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
-		break;
-	case SocketEventType::connection:
+	case fz::socket_event_flag::connection_next:
 		if (error) {
-			if (m_proxyState == handshake)
+			m_pOwner->LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), fz::socket::error_description(error));
+		}
+		break;
+	case fz::socket_event_flag::connection:
+		if (error) {
+			if (m_proxyState == handshake) {
 				m_proxyState = noconn;
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, error);
+			}
+			m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, error);
 		}
 		else {
 			m_pOwner->LogMessage(MessageType::Status, _("Connection with proxy established, performing handshake..."));
 		}
 		break;
-	case SocketEventType::read:
+	case fz::socket_event_flag::read:
 		OnReceive();
 		break;
-	case SocketEventType::write:
+	case fz::socket_event_flag::write:
 		OnSend();
 		break;
-	case SocketEventType::close:
+	case fz::socket_event_flag::close:
 		OnReceive();
 		break;
 	}
 }
 
-void CProxySocket::OnHostAddress(CSocketEventSource*, wxString const& address)
+void CProxySocket::OnHostAddress(socket_event_source*, std::string const& address)
 {
 	m_pOwner->LogMessage(MessageType::Status, _("Connecting to %s..."), address);
 }
 
 void CProxySocket::Detach()
 {
-	if (!m_pSocket)
+	if (!socket_) {
 		return;
+	}
 
-	m_pSocket->SetEventHandler(0);
-	m_pSocket = 0;
+	socket_->set_event_handler(0);
+	socket_ = 0;
 }
 
 void CProxySocket::OnReceive()
 {
 	m_can_read = true;
 
-	if (m_proxyState != handshake)
+	if (m_proxyState != handshake) {
 		return;
+	}
 
 	switch (m_handshakeState)
 	{
@@ -268,28 +267,31 @@ void CProxySocket::OnReceive()
 			char* end = 0;
 			for (int i = 0; i < 2; ++i) {
 				int read;
-				if (!i)
-					read = m_pSocket->Peek(m_pRecvBuffer + m_recvBufferPos, do_read, error);
-				else
-					read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, do_read, error);
+				if (!i) {
+					read = socket_->peek(m_pRecvBuffer + m_recvBufferPos, do_read, error);
+				}
+				else {
+					read = socket_->read(m_pRecvBuffer + m_recvBufferPos, do_read, error);
+				}
 				if (read == -1) {
 					if (error != EAGAIN) {
 						m_proxyState = noconn;
-						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
+						m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, error);
 					}
-					else
+					else {
 						m_can_read = false;
+					}
 					return;
 				}
 				if (!read) {
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				if (m_pSendBuffer) {
 					m_proxyState = noconn;
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming data before request fully sent"));
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pOwner->LogMessage(MessageType::Debug_Warning, L"Incoming data before request fully sent");
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 
@@ -300,73 +302,77 @@ void CProxySocket::OnReceive()
 					if (!end) {
 						if (m_recvBufferPos + read + 1 == m_recvBufferLen) {
 							m_proxyState = noconn;
-							m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming header too large"));
-							m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ENOMEM);
+							m_pOwner->LogMessage(MessageType::Debug_Warning, L"Incoming header too large");
+							m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ENOMEM);
 							return;
 						}
 						do_read = read;
 					}
-					else
+					else {
 						do_read = end - m_pRecvBuffer + 4 - m_recvBufferPos;
+					}
 				}
 				else {
 					if (read != do_read) {
 						m_proxyState = noconn;
-						m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Could not read what got peeked"));
-						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+						m_pOwner->LogMessage(MessageType::Debug_Warning, "Could not read what got peeked");
+						m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 						return;
 					}
 					m_recvBufferPos += read;
 				}
 			}
 
-			if (!end)
+			if (!end) {
 				continue;
+			}
 
 			end = strchr(m_pRecvBuffer, '\r'); // Never fails as old value of end exists and starts with CR, we just look for an earlier case.
 			*end = 0;
-			wxString reply(m_pRecvBuffer, wxConvUTF8);
+			std::wstring const reply = fz::to_wstring_from_utf8(m_pRecvBuffer);
 			m_pOwner->LogMessage(MessageType::Response, _("Proxy reply: %s"), reply);
 
-			if (reply.Left(10) != _T("HTTP/1.1 2") && reply.Left(10) != _T("HTTP/1.0 2")) {
+			if (reply.substr(0, 10) != L"HTTP/1.1 2" && reply.substr(0, 10) != L"HTTP/1.0 2") {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNRESET);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNRESET);
 				return;
 			}
 
 			m_proxyState = conn;
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
+			m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, 0);
 			return;
 		}
 	case socks4_handshake:
 		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int read_error;
-			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, read_error);
+			int read = socket_->read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, read_error);
 			if (read == -1) {
 				if (read_error != EAGAIN) {
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, read_error);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, read_error);
 				}
-				else
+				else {
 					m_can_read = false;
+				}
 				return;
 			}
 
 			if (!read) {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
 			m_recvBufferLen -= read;
 
-			if (m_recvBufferLen)
+			if (m_recvBufferLen) {
 				continue;
+			}
 
 			m_recvBufferPos = 0;
 
 			if (m_pRecvBuffer[1] != 0x5A) {
-				wxString error;
+				std::wstring error;
 				switch (m_pRecvBuffer[1]) {
 					case 0x5B:
 						error = _("Request rejected or failed");
@@ -378,16 +384,16 @@ void CProxySocket::OnReceive()
 						error = _("Request failed - client's identd could not confirm the user ID string");
 						break;
 					default:
-						error.Printf(_("Unassigned error code %d"), (int)(unsigned char) m_pRecvBuffer[1]);
+						error = fz::sprintf(_("Unassigned error code %d"), (int)(unsigned char)m_pRecvBuffer[1]);
 						break;
 				}
 				m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: %s"), error);
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 				return;
 			}
 			m_proxyState = conn;
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
+			m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, 0);
 		}
 		return;
 	case socks5_method:
@@ -395,30 +401,33 @@ void CProxySocket::OnReceive()
 	case socks5_request:
 	case socks5_request_addrtype:
 	case socks5_request_address:
-		if (m_pSendBuffer)
+		if (m_pSendBuffer) {
 			return;
+		}
 		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int error;
-			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, error);
+			int read = socket_->read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, error);
 			if (read == -1) {
 				if (error != EAGAIN) {
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, error);
 				}
-				else
+				else {
 					m_can_read = false;
+				}
 				return;
 			}
 			if (!read) {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
 			m_recvBufferLen -= read;
 
-			if (m_recvBufferLen)
+			if (m_recvBufferLen) {
 				continue;
+			}
 
 			m_recvBufferPos = 0;
 
@@ -426,17 +435,17 @@ void CProxySocket::OnReceive()
 			switch (m_handshakeState) {
 			default:
 				if (m_pRecvBuffer[0] != 5) {
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
+					m_pOwner->LogMessage(MessageType::Error, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				break;
 			case socks5_auth:
 				if (m_pRecvBuffer[0] != 1) {
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
+					m_pOwner->LogMessage(MessageType::Error, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				break;
@@ -458,26 +467,25 @@ void CProxySocket::OnReceive()
 						m_handshakeState = socks5_auth;
 						break;
 					default:
-						m_pOwner->LogMessage(MessageType::Debug_Warning, _("No supported SOCKS5 auth method"));
+						m_pOwner->LogMessage(MessageType::Error, _("No supported SOCKS5 auth method"));
 						m_proxyState = noconn;
-						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+						m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 						return;
 					}
 				}
 				break;
 			case socks5_auth:
-				if (m_pRecvBuffer[1] != 0)
-				{
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy authentication failed"));
+				if (m_pRecvBuffer[1] != 0) {
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy authentication failed"));
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request;
 				break;
 			case socks5_request:
 				if (m_pRecvBuffer[1]) {
-					wxString errorMsg;
+					std::wstring errorMsg;
 					switch (m_pRecvBuffer[1])
 					{
 					case 1:
@@ -505,13 +513,13 @@ void CProxySocket::OnReceive()
 						errorMsg = _("Address type not supported");
 						break;
 					default:
-						errorMsg.Printf(_("Unassigned error code %d"), (int)(unsigned char)m_pRecvBuffer[1]);
+						errorMsg = fz::sprintf(_("Unassigned error code %d"), (int)(unsigned char)m_pRecvBuffer[1]);
 						break;
 					}
 
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: %s"), errorMsg);
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed. Reply from proxy: %s"), errorMsg);
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_addrtype;
@@ -533,9 +541,9 @@ void CProxySocket::OnReceive()
 					m_recvBufferLen = 17;
 					break;
 				default:
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: Unknown address type in CONNECT reply"));
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: Unknown address type in CONNECT reply"));
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_address;
@@ -544,11 +552,11 @@ void CProxySocket::OnReceive()
 				{
 					// We're done
 					m_proxyState = conn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
+					m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::connection, 0);
 					return;
 				}
 			default:
-				wxFAIL;
+				assert(false);
 				break;
 			}
 
@@ -556,71 +564,64 @@ void CProxySocket::OnReceive()
 			{
 			case socks5_auth:
 				{
-					const wxWX2MBbuf user = m_user.mb_str(wxConvUTF8);
-					const wxWX2MBbuf pass = m_pass.mb_str(wxConvUTF8);
-
-					const int userlen = strlen(user);
-					const int passlen = strlen(pass);
-					m_sendBufferLen = userlen + passlen + 3;
+					auto ulen = static_cast<unsigned char>(std::min(m_user.size(), size_t(255)));
+					auto plen = static_cast<unsigned char>(std::min(m_pass.size(), size_t(255)));
+					m_sendBufferLen = ulen + plen + 3;
 					m_pSendBuffer = new char[m_sendBufferLen];
 					m_pSendBuffer[0] = 1;
-					m_pSendBuffer[1] = userlen;
-					memcpy(m_pSendBuffer + 2, (const char*)user, userlen);
-					m_pSendBuffer[userlen + 2] = passlen;
-					memcpy(m_pSendBuffer + userlen + 3, (const char*)pass, passlen);
+					m_pSendBuffer[1] = ulen;
+					memcpy(m_pSendBuffer + 2, m_user.c_str(), ulen);
+					m_pSendBuffer[ulen + 2] = plen;
+					memcpy(m_pSendBuffer + ulen + 3, m_pass.c_str(), plen);
 					m_recvBufferLen = 2;
 				}
 				break;
 			case socks5_request:
 				{
-					const wxWX2MBbuf host = m_host.mb_str(wxConvUTF8);
-					const int hostlen = strlen(host);
-					int addrlen = wxMax(hostlen, 16);
+					std::string host = fz::to_utf8(m_host);
+					size_t addrlen = std::max(host.size(), size_t(16));
 
 					m_pSendBuffer = new char[7 + addrlen];
 					m_pSendBuffer[0] = 5;
 					m_pSendBuffer[1] = 1; // CONNECT
 					m_pSendBuffer[2] = 0; // Reserved
 
-					if (IsIpAddress(m_host)) {
-						// Quite ugly
-						wxString ipv6 = GetIPV6LongForm(m_host);
-						if (!ipv6.empty()) {
-							ipv6.Replace(_T(":"), _T(""));
-							addrlen = 16;
-							for (int i = 0; i < 16; i++)
-								m_pSendBuffer[4 + i] = (DigitHexToDecNum(ipv6[i * 2]) << 4) + DigitHexToDecNum(ipv6[i * 2 + 1]);
-
-							m_pSendBuffer[3] = 4; // IPv6
+					auto const type = fz::get_address_type(host);
+					if (type == fz::address_type::ipv6) {
+						auto ipv6 = fz::get_ipv6_long_form(host);
+						addrlen = 16;
+						for (auto i = 0; i < 16; ++i) {
+							m_pSendBuffer[4 + i] = (fz::hex_char_to_int(ipv6[i * 2 + i / 2]) << 4) + fz::hex_char_to_int(ipv6[i * 2 + 1 + i / 2]);
 						}
-						else {
-							unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
-							int i = 0;
-							memset(buf, 0, 4);
-							for (const wxChar* p = m_host.c_str(); *p && i < 4; p++)
-							{
-								const wxChar& c = *p;
-								if (c == '.')
-								{
-									i++;
-									continue;
-								}
-								buf[i] *= 10;
-								buf[i] += c - '0';
+
+						m_pSendBuffer[3] = 4; // IPv6
+					}
+					else if (type == fz::address_type::ipv4) {
+						unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
+						int i = 0;
+						memset(buf, 0, 4);
+						for (auto p = host.c_str(); *p && i < 4; ++p) {
+							auto const& c = *p;
+							if (c == '.') {
+								++i;
+								continue;
 							}
-
-							addrlen = 4;
-
-							m_pSendBuffer[3] = 1; // IPv4
+							buf[i] *= 10;
+							buf[i] += c - '0';
 						}
+
+						addrlen = 4;
+
+						m_pSendBuffer[3] = 1; // IPv4
 					}
 					else {
 						m_pSendBuffer[3] = 3; // Domain name
-						m_pSendBuffer[4] = hostlen;
-						memcpy(m_pSendBuffer + 5, (const char*)host, hostlen);
-						addrlen = hostlen + 1;
-					}
 
+						auto hlen = static_cast<unsigned char>(std::min(host.size(), size_t(255)));
+						m_pSendBuffer[4] = hlen;
+						memcpy(m_pSendBuffer + 5, host.c_str(), hlen);
+						addrlen = hlen + 1;
+					}
 
 					m_pSendBuffer[addrlen + 4] = (m_port >> 8) & 0xFF; // Port in network order
 					m_pSendBuffer[addrlen + 5] = m_port & 0xFF;
@@ -634,17 +635,18 @@ void CProxySocket::OnReceive()
 				// Nothing to send, we simply need to wait for more data
 				break;
 			default:
-				wxFAIL;
+				assert(false);
 				break;
 			}
-			if (m_pSendBuffer && m_can_write)
+			if (m_pSendBuffer && m_can_write) {
 				OnSend();
+			}
 		}
 		break;
 	default:
 		m_proxyState = noconn;
-		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Unhandled handshake state %d"), m_handshakeState);
-		m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+		m_pOwner->LogMessage(MessageType::Debug_Warning, L"Unhandled handshake state %d", m_handshakeState);
+		m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, ECONNABORTED);
 		return;
 	}
 }
@@ -652,19 +654,21 @@ void CProxySocket::OnReceive()
 void CProxySocket::OnSend()
 {
 	m_can_write = true;
-	if (m_proxyState != handshake || !m_pSendBuffer)
+	if (m_proxyState != handshake || !m_pSendBuffer) {
 		return;
+	}
 
 	for (;;) {
 		int error;
-		int written = m_pSocket->Write(m_pSendBuffer, m_sendBufferLen, error);
+		int written = socket_->write(m_pSendBuffer, m_sendBufferLen, error);
 		if (written == -1) {
 			if (error != EAGAIN) {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
+				m_pEvtHandler->send_event<fz::socket_event>(this, fz::socket_event_flag::close, error);
 			}
-			else
+			else {
 				m_can_write = false;
+			}
 
 			return;
 		}
@@ -673,8 +677,9 @@ void CProxySocket::OnSend()
 			delete [] m_pSendBuffer;
 			m_pSendBuffer = 0;
 
-			if (m_can_read)
+			if (m_can_read) {
 				OnReceive();
+			}
 			return;
 		}
 		memmove(m_pSendBuffer, m_pSendBuffer + written, m_sendBufferLen - written);
@@ -699,3 +704,13 @@ int CProxySocket::Write(const void *, unsigned int, int& error)
 	error = EAGAIN;
 	return -1;
 }
+
+std::wstring CProxySocket::GetUser() const
+{
+	return fz::to_wstring_from_utf8(m_user);
+}
+
+std::wstring CProxySocket::GetPass() const
+{
+	return fz::to_wstring_from_utf8(m_pass);
+}
