@@ -1,12 +1,14 @@
 #include <filezilla.h>
 #include "ratelimiter.h"
 
-#include "event_loop.h"
+#include <libfilezilla/event_handler.hpp>
+
+#include <assert.h>
 
 static int const tickDelay = 250;
 
-CRateLimiter::CRateLimiter(CEventLoop& loop, COptionsBase& options)
-	: CEventHandler(loop)
+CRateLimiter::CRateLimiter(fz::event_loop& loop, COptionsBase& options)
+	: event_handler(loop)
 	, options_(options)
 {
 	RegisterOption(OPTION_SPEEDLIMIT_ENABLE);
@@ -19,7 +21,7 @@ CRateLimiter::CRateLimiter(CEventLoop& loop, COptionsBase& options)
 
 CRateLimiter::~CRateLimiter()
 {
-	RemoveHandler();
+	remove_handler();
 }
 
 int64_t CRateLimiter::GetLimit(rate_direction direction) const
@@ -34,7 +36,7 @@ int64_t CRateLimiter::GetLimit(rate_direction direction) const
 
 void CRateLimiter::AddObject(CRateLimiterObject* pObject)
 {
-	scoped_lock lock(sync_);
+	fz::scoped_lock lock(sync_);
 
 	m_objectList.push_back(pObject);
 
@@ -57,8 +59,9 @@ void CRateLimiter::AddObject(CRateLimiterObject* pObject)
 
 			pObject->m_bytesAvailable[i] = tokens;
 
-			if (!m_timer)
-				m_timer = AddTimer(duration::from_milliseconds(tickDelay), false);
+			if (!m_timer) {
+				m_timer = add_timer(fz::duration::from_milliseconds(tickDelay), false);
+			}
 		}
 		else {
 			pObject->m_bytesAvailable[i] = -1;
@@ -68,7 +71,7 @@ void CRateLimiter::AddObject(CRateLimiterObject* pObject)
 
 void CRateLimiter::RemoveObject(CRateLimiterObject* pObject)
 {
-	scoped_lock lock(sync_);
+	fz::scoped_lock lock(sync_);
 
 	for (auto iter = m_objectList.begin(); iter != m_objectList.end(); ++iter) {
 		if (*iter == pObject) {
@@ -79,8 +82,9 @@ void CRateLimiter::RemoveObject(CRateLimiterObject* pObject)
 				int64_t limit = GetLimit(static_cast<rate_direction>(i));
 				int64_t tokens = limit / (1000 / tickDelay);
 				tokens /= m_objectList.size();
-				if ((*iter)->m_bytesAvailable[i] < tokens)
+				if ((*iter)->m_bytesAvailable[i] < tokens) {
 					m_tokenDebt[i] += tokens - (*iter)->m_bytesAvailable[i];
+				}
 			}
 			m_objectList.erase(iter);
 			break;
@@ -97,23 +101,25 @@ void CRateLimiter::RemoveObject(CRateLimiterObject* pObject)
 	}
 }
 
-void CRateLimiter::OnTimer(timer_id)
+void CRateLimiter::OnTimer(fz::timer_id)
 {
-	scoped_lock lock(sync_);
+	fz::scoped_lock lock(sync_);
 
 	int64_t const limits[2] = { GetLimit(inbound), GetLimit(outbound) };
 
 	for (int i = 0; i < 2; ++i) {
 		m_tokenDebt[i] = 0;
 
-		if (m_objectList.empty())
+		if (m_objectList.empty()) {
 			continue;
+		}
 
 		if (limits[i] == 0) {
 			for (auto iter = m_objectList.begin(); iter != m_objectList.end(); ++iter) {
 				(*iter)->m_bytesAvailable[i] = -1;
-				if ((*iter)->m_waiting[i])
+				if ((*iter)->m_waiting[i]) {
 					m_wakeupList[i].push_back(*iter);
+				}
 			}
 			continue;
 		}
@@ -124,8 +130,9 @@ void CRateLimiter::OnTimer(timer_id)
 		// Get amount of tokens for each object
 		int64_t tokensPerObject = tokens / m_objectList.size();
 
-		if (tokensPerObject == 0)
+		if (tokensPerObject == 0) {
 			tokensPerObject = 1;
+		}
 		tokens = 0;
 
 		// This list will hold all objects which didn't reach maxTokens
@@ -133,7 +140,7 @@ void CRateLimiter::OnTimer(timer_id)
 
 		for (auto iter = m_objectList.begin(); iter != m_objectList.end(); ++iter) {
 			if ((*iter)->m_bytesAvailable[i] == -1) {
-				wxASSERT(!(*iter)->m_waiting[i]);
+				assert(!(*iter)->m_waiting[i]);
 				(*iter)->m_bytesAvailable[i] = tokensPerObject;
 				unsaturatedObjects.push_back(*iter);
 			}
@@ -144,11 +151,13 @@ void CRateLimiter::OnTimer(timer_id)
 					tokens += (*iter)->m_bytesAvailable[i] - maxTokens;
 					(*iter)->m_bytesAvailable[i] = maxTokens;
 				}
-				else
+				else {
 					unsaturatedObjects.push_back(*iter);
+				}
 
-				if ((*iter)->m_waiting[i])
+				if ((*iter)->m_waiting[i]) {
 					m_wakeupList[i].push_back(*iter);
+				}
 			}
 		}
 
@@ -156,8 +165,9 @@ void CRateLimiter::OnTimer(timer_id)
 		// assign to the unsaturated sources
 		while (tokens != 0 && !unsaturatedObjects.empty()) {
 			tokensPerObject = tokens / unsaturatedObjects.size();
-			if (tokensPerObject == 0)
+			if (tokensPerObject == 0) {
 				break;
+			}
 			tokens = 0;
 
 			std::list<CRateLimiterObject*> objects;
@@ -169,8 +179,9 @@ void CRateLimiter::OnTimer(timer_id)
 					tokens += (*iter)->m_bytesAvailable[i] - maxTokens;
 					(*iter)->m_bytesAvailable[i] = maxTokens;
 				}
-				else
+				else {
 					unsaturatedObjects.push_back(*iter);
+				}
 			}
 		}
 	}
@@ -179,22 +190,23 @@ void CRateLimiter::OnTimer(timer_id)
 
 	if (m_objectList.empty() || (limits[inbound] == 0 && limits[outbound] == 0)) {
 		if (m_timer) {
-			StopTimer(m_timer);
+			stop_timer(m_timer);
 			m_timer = 0;
 		}
 	}
 }
 
-void CRateLimiter::WakeupWaitingObjects(scoped_lock & l)
+void CRateLimiter::WakeupWaitingObjects(fz::scoped_lock & l)
 {
 	for (int i = 0; i < 2; ++i) {
 		while (!m_wakeupList[i].empty()) {
 			CRateLimiterObject* pObject = m_wakeupList[i].front();
 			m_wakeupList[i].pop_front();
-			if (!pObject->m_waiting[i])
+			if (!pObject->m_waiting[i]) {
 				continue;
+			}
 
-			wxASSERT(pObject->m_bytesAvailable[i] != 0);
+			assert(pObject->m_bytesAvailable[i] != 0);
 			pObject->m_waiting[i] = false;
 
 			l.unlock(); // Do not hold while executing callback
@@ -224,26 +236,26 @@ int CRateLimiter::GetBucketSize() const
 	return bucket_size;
 }
 
-void CRateLimiter::operator()(CEventBase const& ev)
+void CRateLimiter::operator()(fz::event_base const& ev)
 {
-	if (Dispatch<CTimerEvent>(ev, this, &CRateLimiter::OnTimer)) {
-		return;
-	}
-	Dispatch<CRateLimitChangedEvent>(ev, this, &CRateLimiter::OnRateChanged);
+	fz::dispatch<fz::timer_event, CRateLimitChangedEvent>(ev, this,
+		&CRateLimiter::OnTimer,
+		&CRateLimiter::OnRateChanged);
 }
 
 void CRateLimiter::OnRateChanged()
 {
-	scoped_lock lock(sync_);
+	fz::scoped_lock lock(sync_);
 	if (GetLimit(inbound) > 0 || GetLimit(outbound) > 0) {
-		if (!m_timer)
-			m_timer = AddTimer(duration::from_milliseconds(tickDelay), false);
+		if (!m_timer) {
+			m_timer = add_timer(fz::duration::from_milliseconds(tickDelay), false);
+		}
 	}
 }
 
 void CRateLimiter::OnOptionsChanged(changed_options_t const&)
 {
-	SendEvent<CRateLimitChangedEvent>();
+	send_event<CRateLimitChangedEvent>();
 }
 
 CRateLimiterObject::CRateLimiterObject()
@@ -256,16 +268,18 @@ CRateLimiterObject::CRateLimiterObject()
 
 void CRateLimiterObject::UpdateUsage(CRateLimiter::rate_direction direction, int usedBytes)
 {
-	wxASSERT(usedBytes <= m_bytesAvailable[direction]);
-	if (usedBytes > m_bytesAvailable[direction])
+	assert(usedBytes <= m_bytesAvailable[direction]);
+	if (usedBytes > m_bytesAvailable[direction]) {
 		m_bytesAvailable[direction] = 0;
-	else
+	}
+	else {
 		m_bytesAvailable[direction] -= usedBytes;
+	}
 }
 
 void CRateLimiterObject::Wait(CRateLimiter::rate_direction direction)
 {
-	wxASSERT(m_bytesAvailable[direction] == 0);
+	assert(m_bytesAvailable[direction] == 0);
 	m_waiting[direction] = true;
 }
 
