@@ -3,6 +3,7 @@
 #include "filezillaapp.h"
 #include "sitemanager.h"
 #include "ipcmutex.h"
+#include "state.h"
 #include "themeprovider.h"
 #include "xmlfunctions.h"
 #include "xrc_helper.h"
@@ -25,7 +26,7 @@ EVT_TREE_BEGIN_LABEL_EDIT(XRCID("ID_TREE"), CBookmarksDialog::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(XRCID("ID_TREE"), CBookmarksDialog::OnEndLabelEdit)
 END_EVENT_TABLE()
 
-CNewBookmarkDialog::CNewBookmarkDialog(wxWindow* parent, wxString& site_path, const CServer* server)
+CNewBookmarkDialog::CNewBookmarkDialog(wxWindow* parent, std::wstring& site_path, ServerWithCredentials const* server)
 	: m_parent(parent)
 	, m_site_path(site_path)
 	, m_server(server)
@@ -34,15 +35,18 @@ CNewBookmarkDialog::CNewBookmarkDialog(wxWindow* parent, wxString& site_path, co
 
 int CNewBookmarkDialog::Run(const wxString &local_path, const CServerPath &remote_path)
 {
-	if (!Load(m_parent, _T("ID_NEWBOOKMARK")))
+	if (!Load(m_parent, _T("ID_NEWBOOKMARK"))) {
 		return wxID_CANCEL;
+	}
 
 	xrc_call(*this, "ID_LOCALPATH", &wxTextCtrl::ChangeValue, local_path);
-	if (!remote_path.empty())
+	if (!remote_path.empty()) {
 		xrc_call(*this, "ID_REMOTEPATH", &wxTextCtrl::ChangeValue, remote_path.GetPath());
+	}
 
-	if (!m_server)
+	if (!m_server) {
 		xrc_call(*this, "ID_TYPE_SITE", &wxRadioButton::Enable, false);
+	}
 
 	return ShowModal();
 }
@@ -62,10 +66,10 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 
 	CServerPath remote_path;
 	if (!remote_path_raw.empty()) {
-		if (!global && m_server)
-			remote_path.SetType(m_server->GetType());
-		if (!remote_path.SetPath(remote_path_raw))
-		{
+		if (!global && m_server) {
+			remote_path.SetType(m_server->server.GetType());
+		}
+		if (!remote_path.SetPath(remote_path_raw.ToStdWstring())) {
 			wxMessageBoxEx(_("Could not parse remote path."), _("New bookmark"), wxICON_EXCLAMATION);
 			return;
 		}
@@ -85,11 +89,14 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 	bool const comparison = xrc_call(*this, "ID_COMPARISON", &wxCheckBox::GetValue);
 
 	if (!global && m_server) {
-		std::list<wxString> bookmarks;
-
-		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks)) {
-			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
+		std::unique_ptr<Site> site;
+		if (!m_site_path.empty()) {
+			site = CSiteManager::GetSiteByPath(m_site_path).first;
+		}
+		if (!site) {
+			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES) {
 				return;
+			}
 
 			m_site_path = CSiteManager::AddServer(*m_server);
 			if (m_site_path.empty()) {
@@ -98,10 +105,12 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 				return;
 			}
 		}
-		for (std::list<wxString>::const_iterator iter = bookmarks.begin(); iter != bookmarks.end(); ++iter) {
-			if (*iter == name) {
-				wxMessageBoxEx(_("A bookmark with the entered name already exists. Please enter an unused name."), _("New bookmark"), wxICON_EXCLAMATION, this);
-				return;
+		else {
+			for (auto const& bookmark : site->m_bookmarks) {
+				if (bookmark.m_name == name) {
+					wxMessageBoxEx(_("A bookmark with the entered name already exists. Please enter an unused name."), _("New bookmark"), wxICON_EXCLAMATION, this);
+					return;
+				}
 			}
 		}
 
@@ -110,8 +119,9 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 		EndModal(wxID_OK);
 	}
 	else {
-		if (!CBookmarksDialog::AddBookmark(name, local_path, remote_path, sync, comparison))
+		if (!CBookmarksDialog::AddBookmark(name, local_path, remote_path, sync, comparison)) {
 			return;
+		}
 
 		EndModal(wxID_OK);
 	}
@@ -140,13 +150,13 @@ public:
 	{
 	}
 
-	wxString m_local_dir;
+	std::wstring m_local_dir;
 	CServerPath m_remote_dir;
 	bool m_sync{};
 	bool m_comparison{};
 };
 
-CBookmarksDialog::CBookmarksDialog(wxWindow* parent, wxString& site_path, const CServer* server)
+CBookmarksDialog::CBookmarksDialog(wxWindow* parent, std::wstring& site_path, ServerWithCredentials const* server)
 	: m_parent(parent)
 	, m_site_path(site_path)
 	, m_server(server)
@@ -159,39 +169,44 @@ void CBookmarksDialog::LoadGlobalBookmarks()
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
 
-	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
+	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString name;
 		wxString local_dir;
-		wxString remote_dir_raw;
+		std::wstring remote_dir_raw;
 		CServerPath remote_dir;
 
-		name = GetTextElement(pBookmark, "Name");
-		if (name.empty())
+		name = GetTextElement(bookmark, "Name");
+		if (name.empty()) {
 			continue;
-
-		local_dir = GetTextElement(pBookmark, "LocalDir");
-		remote_dir_raw = GetTextElement(pBookmark, "RemoteDir");
-		if (!remote_dir_raw.empty()) {
-			if (!remote_dir.SetSafePath(remote_dir_raw))
-				continue;
 		}
-		if (local_dir.empty() && remote_dir.empty())
+
+		local_dir = GetTextElement(bookmark, "LocalDir");
+		remote_dir_raw = GetTextElement(bookmark, "RemoteDir");
+		if (!remote_dir_raw.empty()) {
+			if (!remote_dir.SetSafePath(remote_dir_raw)) {
+				continue;
+			}
+		}
+		if (local_dir.empty() && remote_dir.empty()) {
 			continue;
+		}
 
 		bool sync;
-		if (local_dir.empty() || remote_dir.empty())
+		if (local_dir.empty() || remote_dir.empty()) {
 			sync = false;
-		else
-			sync = GetTextElementBool(pBookmark, "SyncBrowsing");
+		}
+		else {
+			sync = GetTextElementBool(bookmark, "SyncBrowsing");
+		}
 
-		bool const comparison = GetTextElementBool(pBookmark, "DirectoryComparison");
+		bool const comparison = GetTextElementBool(bookmark, "DirectoryComparison");
 
 		CBookmarkItemData *data = new CBookmarkItemData(local_dir, remote_dir, sync, comparison);
 		m_pTree->AppendItem(m_bookmarks_global, name, 1, 1, data);
@@ -202,25 +217,18 @@ void CBookmarksDialog::LoadGlobalBookmarks()
 
 void CBookmarksDialog::LoadSiteSpecificBookmarks()
 {
-	if (m_site_path.empty())
+	if (m_site_path.empty()) {
 		return;
+	}
 
-	std::list<wxString> bookmarks;
-
-	if (!CSiteManager::GetBookmarks(m_site_path, bookmarks))
+	auto const site = CSiteManager::GetSiteByPath(m_site_path).first;
+	if (!site) {
 		return;
+	}
 
-	for (std::list<wxString>::const_iterator iter = bookmarks.begin(); iter != bookmarks.end(); ++iter) {
-		wxString path = *iter;
-		path.Replace(_T("\\"), _T("\\\\"));
-		path.Replace(_T("/"), _T("\\/"));
-		path = m_site_path + _T("/") + path;
-
-		std::unique_ptr<CSiteManagerItemData_Site> data = CSiteManager::GetSiteByPath(path);
-		if (data) {
-			CBookmarkItemData* new_data = new CBookmarkItemData(data->m_localDir, data->m_remoteDir, data->m_sync, data->m_comparison);
-			m_pTree->AppendItem(m_bookmarks_site, *iter, 1, 1, new_data);
-		}
+	for (auto const& bookmark : site->m_bookmarks) {
+		CBookmarkItemData* new_data = new CBookmarkItemData(bookmark.m_localDir, bookmark.m_remoteDir, bookmark.m_sync, bookmark.m_comparison);
+		m_pTree->AppendItem(m_bookmarks_site, bookmark.m_name, 1, 1, new_data);
 	}
 
 	m_pTree->SortChildren(m_bookmarks_site);
@@ -228,13 +236,15 @@ void CBookmarksDialog::LoadSiteSpecificBookmarks()
 
 int CBookmarksDialog::Run()
 {
-	if (!Load(m_parent, _T("ID_BOOKMARKS")))
+	if (!Load(m_parent, _T("ID_BOOKMARKS"))) {
 		return wxID_CANCEL;
+	}
 
 	// Now create the imagelist for the site tree
 	m_pTree = XRCCTRL(*this, "ID_TREE", wxTreeCtrl);
-	if (!m_pTree)
+	if (!m_pTree) {
 		return false;
+	}
 
 	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
 	wxImageList* pImageList = new wxImageList(s.x, s.y);
@@ -280,8 +290,8 @@ void CBookmarksDialog::SaveGlobalBookmarks()
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxString msg = file.GetError() + _T("\n\n") + _("The global bookmarks could not be saved.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
@@ -289,10 +299,10 @@ void CBookmarksDialog::SaveGlobalBookmarks()
 	}
 
 	{
-		TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark");
-		while (pBookmark) {
-			pDocument->RemoveChild(pBookmark);
-			pBookmark = pDocument->FirstChildElement("Bookmark");
+		auto bookmark = element.child("Bookmark");
+		while (bookmark) {
+			element.remove_child(bookmark);
+			bookmark = element.child("Bookmark");
 		}
 	}
 
@@ -301,46 +311,56 @@ void CBookmarksDialog::SaveGlobalBookmarks()
 		CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(child);
 		wxASSERT(data);
 
-		TiXmlElement *pBookmark = pDocument->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
-		AddTextElement(pBookmark, "Name", m_pTree->GetItemText(child));
-		if (!data->m_local_dir.empty())
-			AddTextElement(pBookmark, "LocalDir", data->m_local_dir);
-		if (!data->m_remote_dir.empty())
-			AddTextElement(pBookmark, "RemoteDir", data->m_remote_dir.GetSafePath());
-		if (data->m_sync)
-			AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
-		if (data->m_comparison)
-			AddTextElementRaw(pBookmark, "DirectoryComparison", "1");
+		auto bookmark = element.append_child("Bookmark");
+		AddTextElement(bookmark, "Name", m_pTree->GetItemText(child).ToStdWstring());
+		if (!data->m_local_dir.empty()) {
+			AddTextElement(bookmark, "LocalDir", data->m_local_dir);
+		}
+		if (!data->m_remote_dir.empty()) {
+			AddTextElement(bookmark, "RemoteDir", data->m_remote_dir.GetSafePath());
+		}
+		if (data->m_sync) {
+			AddTextElementUtf8(bookmark, "SyncBrowsing", "1");
+		}
+		if (data->m_comparison) {
+			AddTextElementUtf8(bookmark, "DirectoryComparison", "1");
+		}
 	}
 
 	if (!file.Save(false)) {
 		wxString msg = wxString::Format(_("Could not write \"%s\", the global bookmarks could no be saved: %s"), file.GetFileName(), file.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
+
+	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_GLOBALBOOKMARKS);
 }
 
 void CBookmarksDialog::SaveSiteSpecificBookmarks()
 {
-	if (m_site_path.empty())
+	if (m_site_path.empty()) {
 		return;
+	}
 
-	if (!CSiteManager::ClearBookmarks(m_site_path))
-			return;
+	if (!CSiteManager::ClearBookmarks(m_site_path)) {
+		return;
+	}
 
 	wxTreeItemIdValue cookie;
 	for (wxTreeItemId child = m_pTree->GetFirstChild(m_bookmarks_site, cookie); child.IsOk(); child = m_pTree->GetNextChild(m_bookmarks_site, cookie)) {
 		CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(child);
 		wxASSERT(data);
 
-		if (!CSiteManager::AddBookmark(m_site_path, m_pTree->GetItemText(child), data->m_local_dir, data->m_remote_dir, data->m_sync, data->m_comparison))
+		if (!CSiteManager::AddBookmark(m_site_path, m_pTree->GetItemText(child), data->m_local_dir, data->m_remote_dir, data->m_sync, data->m_comparison)) {
 			return;
+		}
 	}
 }
 
 void CBookmarksDialog::OnOK(wxCommandEvent&)
 {
-	if (!Verify())
+	if (!Verify()) {
 		return;
+	}
 	UpdateBookmark();
 
 	SaveGlobalBookmarks();
@@ -352,12 +372,14 @@ void CBookmarksDialog::OnOK(wxCommandEvent&)
 void CBookmarksDialog::OnBrowse(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(item);
-	if (!data)
+	if (!data) {
 		return;
+	}
 
 	wxTextCtrl *pText = XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl);
 
@@ -369,8 +391,9 @@ void CBookmarksDialog::OnBrowse(wxCommandEvent&)
 
 void CBookmarksDialog::OnSelChanging(wxTreeEvent& event)
 {
-	if (m_is_deleting)
+	if (m_is_deleting) {
 		return;
+	}
 
 	if (!Verify()) {
 		event.Veto();
@@ -388,36 +411,44 @@ void CBookmarksDialog::OnSelChanged(wxTreeEvent&)
 bool CBookmarksDialog::Verify()
 {
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item)
+	if (!item) {
 		return true;
+	}
 
 	CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(item);
-	if (!data)
+	if (!data) {
 		return true;
+	}
 
-	const CServer *server;
-	if (m_pTree->GetItemParent(item) == m_bookmarks_site)
+	ServerWithCredentials const* server;
+	if (m_pTree->GetItemParent(item) == m_bookmarks_site) {
 		server = m_server;
-	else
+	}
+	else {
 		server = 0;
+	}
 
 	wxString const remotePathRaw = xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::GetValue);
 	if (!remotePathRaw.empty()) {
 		CServerPath remotePath;
-		if (server)
-			remotePath.SetType(server->GetType());
-		if (!remotePath.SetPath(remotePathRaw)) {
+		if (server) {
+			remotePath.SetType(server->server.GetType());
+		}
+		if (!remotePath.SetPath(remotePathRaw.ToStdWstring())) {
 			xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::SetFocus);
 			if (server) {
 				wxString msg;
-				if (server->GetType() != DEFAULT)
-					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the current site's servertype (%s)."), server->GetNameFromServerType(server->GetType()));
-				else
+				if (server->server.GetType() != DEFAULT) {
+					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the current site's servertype (%s)."), CServer::GetNameFromServerType(server->server.GetType()));
+				}
+				else {
 					msg = _("Remote path cannot be parsed. Make sure it is a valid absolute path.");
+				}
 				wxMessageBoxEx(msg);
 			}
-			else
+			else {
 				wxMessageBoxEx(_("Remote path cannot be parsed. Make sure it is a valid absolute path."));
+			}
 			return false;
 		}
 	}
@@ -442,24 +473,29 @@ bool CBookmarksDialog::Verify()
 void CBookmarksDialog::UpdateBookmark()
 {
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	CBookmarkItemData *data = (CBookmarkItemData *)m_pTree->GetItemData(item);
-	if (!data)
+	if (!data) {
 		return;
+	}
 
-	const CServer *server;
-	if (m_pTree->GetItemParent(item) == m_bookmarks_site)
+	ServerWithCredentials const* server;
+	if (m_pTree->GetItemParent(item) == m_bookmarks_site) {
 		server = m_server;
-	else
+	}
+	else {
 		server = 0;
+	}
 
 	wxString const remotePathRaw = xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::GetValue);
 	if (!remotePathRaw.empty()) {
-		if (server)
-			data->m_remote_dir.SetType(server->GetType());
-		data->m_remote_dir.SetPath(remotePathRaw);
+		if (server) {
+			data->m_remote_dir.SetType(server->server.GetType());
+		}
+		data->m_remote_dir.SetPath(remotePathRaw.ToStdWstring());
 	}
 
 	data->m_local_dir = xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetValue);
@@ -506,23 +542,31 @@ void CBookmarksDialog::DisplayBookmark()
 
 void CBookmarksDialog::OnNewBookmark(wxCommandEvent&)
 {
-	if (!Verify())
+	if (!Verify()) {
 		return;
+	}
 	UpdateBookmark();
 
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item)
+	if (!item) {
 		item = m_bookmarks_global;
+	}
 
-	if (m_pTree->GetItemData(item))
+	if (m_pTree->GetItemData(item)) {
 		item = m_pTree->GetItemParent(item);
+	}
 
 	if (item == m_bookmarks_site) {
-		std::list<wxString> bookmarks;
 
-		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks)) {
-			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
+		std::unique_ptr<Site> site;
+		if (!m_site_path.empty()) {
+			site = CSiteManager::GetSiteByPath(m_site_path).first;
+		}
+
+		if (!site) {
+			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES) {
 				return;
+			}
 
 			m_site_path = CSiteManager::AddServer(*m_server);
 			if (m_site_path.empty()) {
@@ -549,8 +593,9 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent&)
 
 			child = m_pTree->GetNextChild(item, cookie);
 		}
-		if (!found)
+		if (!found) {
 			break;
+		}
 
 		newName = _("New bookmark") + wxString::Format(_T(" %d"), index++);
 	}
@@ -564,8 +609,9 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent&)
 void CBookmarksDialog::OnRename(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
+	if (!item || item == m_bookmarks_global || item == m_bookmarks_site) {
 		return;
+	}
 
 	m_pTree->EditLabel(item);
 }
@@ -573,8 +619,9 @@ void CBookmarksDialog::OnRename(wxCommandEvent&)
 void CBookmarksDialog::OnDelete(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
+	if (!item || item == m_bookmarks_global || item == m_bookmarks_site) {
 		return;
+	}
 
 	wxTreeItemId parent = m_pTree->GetItemParent(item);
 
@@ -587,15 +634,18 @@ void CBookmarksDialog::OnDelete(wxCommandEvent&)
 void CBookmarksDialog::OnCopy(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
-	if (!item.IsOk())
+	if (!item.IsOk()) {
 		return;
+	}
 
-	if (!Verify())
+	if (!Verify()) {
 		return;
+	}
 
 	CBookmarkItemData* data = static_cast<CBookmarkItemData *>(m_pTree->GetItemData(item));
-	if (!data)
+	if (!data) {
 		return;
+	}
 
 	UpdateBookmark();
 
@@ -619,8 +669,9 @@ void CBookmarksDialog::OnCopy(wxCommandEvent&)
 
 			child = m_pTree->GetNextChild(parent, cookie);
 		}
-		if (!found)
+		if (!found) {
 			break;
+		}
 
 		newName = wxString::Format(_("Copy (%d) of %s"), index++, oldName);
 	}
@@ -651,8 +702,9 @@ void CBookmarksDialog::OnBeginLabelEdit(wxTreeEvent& event)
 
 void CBookmarksDialog::OnEndLabelEdit(wxTreeEvent& event)
 {
-	if (event.IsEditCancelled())
+	if (event.IsEditCancelled()) {
 		return;
+	}
 
 	wxTreeItemId item = event.GetItem();
 	if (item != m_pTree->GetSelection()) {
@@ -673,8 +725,9 @@ void CBookmarksDialog::OnEndLabelEdit(wxTreeEvent& event)
 
 	wxTreeItemIdValue cookie;
 	for (wxTreeItemId child = m_pTree->GetFirstChild(parent, cookie); child.IsOk(); child = m_pTree->GetNextChild(parent, cookie)) {
-		if (child == item)
+		if (child == item) {
 			continue;
+		}
 		if (!name.CmpNoCase(m_pTree->GetItemText(child))) {
 			wxMessageBoxEx(_("Name already exists"), _("Cannot rename entry"), wxICON_EXCLAMATION, this);
 			event.Veto();
@@ -685,36 +738,39 @@ void CBookmarksDialog::OnEndLabelEdit(wxTreeEvent& event)
 	m_pTree->SortChildren(parent);
 }
 
-bool CBookmarksDialog::GetBookmarks(std::list<wxString> &bookmarks)
+bool CBookmarksDialog::GetGlobalBookmarks(std::list<wxString> &bookmarks)
 {
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
+	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString name;
 		wxString local_dir;
-		wxString remote_dir_raw;
+		std::wstring remote_dir_raw;
 		CServerPath remote_dir;
 
-		name = GetTextElement(pBookmark, "Name");
-		if (name.empty())
+		name = GetTextElement(bookmark, "Name");
+		if (name.empty()) {
 			continue;
-
-		local_dir = GetTextElement(pBookmark, "LocalDir");
-		remote_dir_raw = GetTextElement(pBookmark, "RemoteDir");
-		if (!remote_dir_raw.empty()) {
-			if (!remote_dir.SetSafePath(remote_dir_raw))
-				continue;
 		}
-		if (local_dir.empty() && remote_dir.empty())
+
+		local_dir = GetTextElement(bookmark, "LocalDir");
+		remote_dir_raw = GetTextElement(bookmark, "RemoteDir");
+		if (!remote_dir_raw.empty()) {
+			if (!remote_dir.SetSafePath(remote_dir_raw)) {
+				continue;
+			}
+		}
+		if (local_dir.empty() && remote_dir.empty()) {
 			continue;
+		}
 
 		bookmarks.push_back(name);
 	}
@@ -727,34 +783,39 @@ bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CS
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
-		wxString remote_dir_raw;
+	for (auto bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
+		std::wstring remote_dir_raw;
 
-		if (name != GetTextElement(pBookmark, "Name"))
+		if (name != GetTextElement(bookmark, "Name")) {
 			continue;
-
-		local_dir = GetTextElement(pBookmark, "LocalDir");
-		remote_dir_raw = GetTextElement(pBookmark, "RemoteDir");
-		if (!remote_dir_raw.empty()) {
-			if (!remote_dir.SetSafePath(remote_dir_raw))
-				return false;
 		}
-		if (local_dir.empty() && remote_dir_raw.empty())
+
+		local_dir = GetTextElement(bookmark, "LocalDir");
+		remote_dir_raw = GetTextElement(bookmark, "RemoteDir");
+		if (!remote_dir_raw.empty()) {
+			if (!remote_dir.SetSafePath(remote_dir_raw)) {
+				return false;
+			}
+		}
+		if (local_dir.empty() && remote_dir_raw.empty()) {
 			return false;
+		}
 
-		if (local_dir.empty() || remote_dir_raw.empty())
+		if (local_dir.empty() || remote_dir_raw.empty()) {
 			sync = false;
-		else
-			sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
+		}
+		else {
+			sync = GetTextElementBool(bookmark, "SyncBrowsing", false);
+		}
 
-		comparison = GetTextElementBool(pBookmark, "DirectoryComparison", false);
+		comparison = GetTextElementBool(bookmark, "DirectoryComparison", false);
 		return true;
 	}
 
@@ -764,50 +825,58 @@ bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CS
 
 bool CBookmarksDialog::AddBookmark(const wxString &name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
 {
-	if (local_dir.empty() && remote_dir.empty())
+	if (local_dir.empty() && remote_dir.empty()) {
 		return false;
-	if ((local_dir.empty() || remote_dir.empty()) && sync)
+	}
+	if ((local_dir.empty() || remote_dir.empty()) && sync) {
 		return false;
+	}
 
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
 	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument) {
+	auto element = file.Load();
+	if (!element) {
 		wxString msg = file.GetError() + _T("\n\n") + _("The bookmark could not be added.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
-	TiXmlElement *pInsertBefore = 0;
-	TiXmlElement *pBookmark;
-	for (pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
+	pugi::xml_node bookmark, insertBefore;
+	for (bookmark = element.child("Bookmark"); bookmark; bookmark = bookmark.next_sibling("Bookmark")) {
 		wxString remote_dir_raw;
 
-		wxString old_name = GetTextElement(pBookmark, "Name");
+		wxString old_name = GetTextElement(bookmark, "Name");
 
 		if (!name.CmpNoCase(old_name)) {
 			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
-		if (name < old_name && !pInsertBefore)
-			pInsertBefore = pBookmark;
+		if (name < old_name && !insertBefore) {
+			insertBefore = bookmark;
+		}
 	}
 
-	if (pInsertBefore)
-		pBookmark = pDocument->InsertBeforeChild(pInsertBefore, TiXmlElement("Bookmark"))->ToElement();
-	else
-		pBookmark = pDocument->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
-	AddTextElement(pBookmark, "Name", name);
-	if (!local_dir.empty())
-		AddTextElement(pBookmark, "LocalDir", local_dir);
-	if (!remote_dir.empty())
-		AddTextElement(pBookmark, "RemoteDir", remote_dir.GetSafePath());
-	if (sync)
-		AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
-	if (comparison)
-		AddTextElementRaw(pBookmark, "DirectoryComparison", "1");
+	if (insertBefore) {
+		bookmark = element.insert_child_before("Bookmark", insertBefore);
+	}
+	else {
+		bookmark = element.append_child("Bookmark");
+	}
+	AddTextElement(bookmark, "Name", name.ToStdWstring());
+	if (!local_dir.empty()) {
+		AddTextElement(bookmark, "LocalDir", local_dir.ToStdWstring());
+	}
+	if (!remote_dir.empty()) {
+		AddTextElement(bookmark, "RemoteDir", remote_dir.GetSafePath());
+	}
+	if (sync) {
+		AddTextElementUtf8(bookmark, "SyncBrowsing", "1");
+	}
+	if (comparison) {
+		AddTextElementUtf8(bookmark, "DirectoryComparison", "1");
+	}
 
 	if (!file.Save(false)) {
 		wxString msg = wxString::Format(_("Could not write \"%s\", the bookmark could not be added: %s"), file.GetFileName(), file.GetError());
@@ -815,5 +884,7 @@ bool CBookmarksDialog::AddBookmark(const wxString &name, const wxString &local_d
 		return false;
 	}
 
+	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_GLOBALBOOKMARKS);
+
 	return true;
 }
