@@ -1,5 +1,5 @@
-#ifndef __DNDOBJECTS_H__
-#define __DNDOBJECTS_H__
+#ifndef FILEZILLA_INTERFACE_DNDOBJECTS_HEADER
+#define FILEZILLA_INTERFACE_DNDOBJECTS_HEADER
 
 #ifdef __WXMSW__
 #define FZ3_USESHELLEXT 1
@@ -9,9 +9,11 @@
 
 #include "xmlfunctions.h"
 
+#include <wx/dnd.h>
+
 #include <memory>
 
-class wxRemoteDataFormat : public wxDataFormat
+class wxRemoteDataFormat final : public wxDataFormat
 {
 public:
 	wxRemoteDataFormat()
@@ -20,10 +22,10 @@ public:
 	}
 };
 
-class CRemoteDataObject : public wxDataObjectSimple
+class CRemoteDataObject final : public wxDataObjectSimple
 {
 public:
-	CRemoteDataObject(const CServer& server, const CServerPath& path);
+	CRemoteDataObject(ServerWithCredentials const& server, const CServerPath& path);
 	CRemoteDataObject();
 
 	virtual size_t GetDataSize() const;
@@ -36,13 +38,13 @@ public:
 
 	bool DidSendData() const { return m_didSendData; }
 
-	const CServer& GetServer() const { return m_server; }
+	ServerWithCredentials const& GetServer() const { return server_; }
 	const CServerPath& GetServerPath() const { return m_path; }
 	int GetProcessId() const { return m_processId; }
 
 	struct t_fileInfo
 	{
-		wxString name;
+		std::wstring name;
 		bool dir;
 		int64_t size;
 		bool link;
@@ -53,12 +55,12 @@ public:
 	void AddFile(const wxString& name, bool dir, int64_t size, bool link);
 
 protected:
-	CServer m_server;
+	ServerWithCredentials server_;
 	CServerPath m_path;
 
 	mutable CXmlFile m_xmlFile;
 
-	bool m_didSendData;
+	bool m_didSendData{};
 
 	int m_processId;
 
@@ -71,11 +73,11 @@ protected:
 
 // This class checks if the shell extension is installed and
 // communicates with it.
-class CShellExtensionInterface
+class CShellExtensionInterface final
 {
 public:
 	CShellExtensionInterface();
-	virtual ~CShellExtensionInterface();
+	~CShellExtensionInterface();
 
 	bool IsLoaded() const { return m_shellExtension != 0; }
 
@@ -94,9 +96,9 @@ protected:
 	HANDLE m_hMutex;
 	HANDLE m_hMapping;
 
-	wxString m_dragDirectory;
+	std::wstring m_dragDirectory;
 };
 
-#endif //__WXMSW__
+#endif
 
-#endif //__DNDOBJECTS_H__
+#endif
