@@ -100,7 +100,7 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 				delete pItem;
 
 			if (iter - m_children.begin() - m_removed_at_front <= 10) {
-				m_removed_at_front++;
+				++m_removed_at_front;
 				unsigned int end = iter - m_children.begin();
 				int c = 0;
 				for (int i = end; i >= m_removed_at_front; i--, c++)
@@ -121,7 +121,7 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 				delete *iter;
 
 				if (iter - m_children.begin() - m_removed_at_front <= 10) {
-					m_removed_at_front++;
+					++m_removed_at_front;
 					unsigned int end = iter - m_children.begin();
 					for (int i = end; i >= m_removed_at_front; i--)
 						m_children[i] = m_children[i - 1];
@@ -135,13 +135,19 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 	};
 
 	if (forward) {
-		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end() && !deleted; ++iter) {
+		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter ) {
 			doRemove(iter);
+			if (deleted) {
+				break;
+			}
 		}
 	}
 	else {
-		for (auto iter = m_children.rbegin(); iter != m_children.rend() - m_removed_at_front && !deleted; ++iter) {
+		for (auto iter = m_children.rbegin(); iter != m_children.rend() - m_removed_at_front; ++iter ) {
 			doRemove(iter.base() - 1);
+			if (deleted) {
+				break;
+			}
 		}
 	}
 
@@ -215,11 +221,11 @@ int CQueueItem::GetItemIndex() const
 }
 
 CFileItem::CFileItem(CServerItem* parent, bool queued, bool download,
-					 const wxString& sourceFile, const wxString& targetFile,
-					 const CLocalPath& localPath, const CServerPath& remotePath, int64_t size)
+					 std::wstring const& sourceFile, std::wstring const& targetFile,
+					 CLocalPath const& localPath, CServerPath const& remotePath, int64_t size)
 	: CQueueItem(parent)
 	, m_sourceFile(sourceFile)
-	, m_targetFile(targetFile.empty() ? CSparseOptional<fzstring>() : CSparseOptional<fzstring>(to_fzstring(targetFile)))
+	, m_targetFile(targetFile.empty() ? fz::sparse_optional<std::wstring>() : fz::sparse_optional<std::wstring>(targetFile))
 	, m_localPath(localPath)
 	, m_remotePath(remotePath)
 	, m_size(size)
@@ -273,30 +279,36 @@ void CFileItem::SetActive(const bool active)
 
 void CFileItem::SaveItem(pugi::xml_node& element) const
 {
-	if (m_edit != CEditHandler::none || !element)
+	if (m_edit != CEditHandler::none || !element) {
 		return;
+	}
 
 	auto file = element.append_child("File");
 
 	AddTextElement(file, "LocalFile", m_localPath.GetPath() + GetLocalFile());
 	AddTextElement(file, "RemoteFile", GetRemoteFile());
 	AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
-	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
-	if (m_size != -1)
+	AddTextElementUtf8(file, "Download", Download() ? "1" : "0");
+	if (m_size != -1) {
 		AddTextElement(file, "Size", m_size);
-	if (m_errorCount)
+	}
+	if (m_errorCount) {
 		AddTextElement(file, "ErrorCount", m_errorCount);
-	if (m_priority != QueuePriority::normal)
+	}
+	if (m_priority != QueuePriority::normal) {
 		AddTextElement(file, "Priority", static_cast<int>(m_priority));
-	AddTextElementRaw(file, "DataType", Ascii() ? "0" : "1");
-	if (m_defaultFileExistsAction != CFileExistsNotification::unknown)
+	}
+	AddTextElementUtf8(file, "DataType", Ascii() ? "0" : "1");
+	if (m_defaultFileExistsAction != CFileExistsNotification::unknown) {
 		AddTextElement(file, "OverwriteAction", m_defaultFileExistsAction);
+	}
 }
 
 bool CFileItem::TryRemoveAll()
 {
-	if (!IsActive())
+	if (!IsActive()) {
 		return true;
+	}
 
 	set_pending_remove(true);
 	return false;
@@ -304,10 +316,12 @@ bool CFileItem::TryRemoveAll()
 
 void CFileItem::SetTargetFile(wxString const& file)
 {
-	if (!file.empty() && file != m_sourceFile)
-		m_targetFile = CSparseOptional<fzstring>(to_fzstring(file));
-	else
+	if (!file.empty() && file != m_sourceFile) {
+		m_targetFile = fz::sparse_optional<std::wstring>(to_wstring(file));
+	}
+	else {
 		m_targetFile.clear();
+	}
 }
 
 void CFileItem::SetStatusMessage(CFileItem::Status status)
@@ -337,13 +351,13 @@ wxString const& CFileItem::GetStatusMessage() const
 	return statusTexts[m_status];
 }
 
-CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CLocalPath& localPath)
-	: CFileItem(parent, queued, true, wxEmptyString, wxEmptyString, localPath, CServerPath(), -1)
+CFolderItem::CFolderItem(CServerItem* parent, bool queued, CLocalPath const& localPath)
+	: CFileItem(parent, queued, true, std::wstring(), std::wstring(), localPath, CServerPath(), -1)
 {
 }
 
-CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CServerPath& remotePath, const wxString& remoteFile)
-	: CFileItem(parent, queued, false, _T(""), remoteFile, CLocalPath(), remotePath, -1)
+CFolderItem::CFolderItem(CServerItem* parent, bool queued, CServerPath const& remotePath, std::wstring const& remoteFile)
+	: CFileItem(parent, queued, false, std::wstring(), remoteFile, CLocalPath(), remotePath, -1)
 {
 }
 
@@ -351,26 +365,29 @@ void CFolderItem::SaveItem(pugi::xml_node& element) const
 {
 	auto file = element.append_child("Folder");
 
-	if (Download())
+	if (Download()) {
 		AddTextElement(file, "LocalFile", GetLocalPath().GetPath() + GetLocalFile());
+	}
 	else {
 		AddTextElement(file, "RemoteFile", GetRemoteFile());
 		AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	}
-	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
+	AddTextElementUtf8(file, "Download", Download() ? "1" : "0");
 }
 
 void CFolderItem::SetActive(const bool active)
 {
-	if (active)
+	if (active) {
 		flags |= flag_active;
-	else
+	}
+	else {
 		flags &= ~flag_active;
+	}
 }
 
-CServerItem::CServerItem(const CServer& server)
+CServerItem::CServerItem(ServerWithCredentials const& server)
 	: m_activeCount(0)
-	, m_server(server)
+	, server_(server)
 {
 }
 
@@ -378,14 +395,14 @@ CServerItem::~CServerItem()
 {
 }
 
-const CServer& CServerItem::GetServer() const
+ServerWithCredentials const& CServerItem::GetServer() const
 {
-	return m_server;
+	return server_;
 }
 
 wxString CServerItem::GetName() const
 {
-	return m_server.FormatServer();
+	return server_.Format(ServerFormat::with_user_and_optional_port);
 }
 
 void CServerItem::AddChild(CQueueItem* pItem)
@@ -451,11 +468,6 @@ void CServerItem::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteA
 				continue;
 			pFileItem->m_defaultFileExistsAction = action;
 		}
-		else if (pItem->GetType() == QueueItemType::FolderScan) {
-			if (direction == TransferDirection::download)
-				continue;
-			((CFolderScanItem *)pItem)->m_defaultFileExistsAction = action;
-		}
 	}
 }
 
@@ -567,7 +579,7 @@ bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 	if (removed) {
 		m_maxCachedIndex = -1;
 	}
-	
+
 	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()) - m_removed_at_front);
 	wxASSERT(((m_children.size() - m_removed_at_front) != 0) == (m_visibleOffspring != 0));
 
@@ -613,14 +625,15 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 
 void CServerItem::SaveItem(pugi::xml_node& element) const
 {
-	auto server = element.append_child("Server");
-	SetServer(server, m_server);
+	auto server_node = element.append_child("Server");
+	SetServer(server_node, server_);
 
-	for (auto iter = m_children.cbegin() + m_removed_at_front; iter != m_children.cend(); ++iter)
-		(*iter)->SaveItem(server);
+	for (auto iter = m_children.cbegin() + m_removed_at_front; iter != m_children.cend(); ++iter) {
+		(*iter)->SaveItem(server_node);
+	}
 }
 
-int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
+int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles) const
 {
 	int64_t totalSize = 0;
 	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
@@ -628,10 +641,12 @@ int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, i
 			const std::deque<CFileItem*>& fileList = m_fileList[j][i];
 			for (auto const& item : fileList) {
 				int64_t size = item->GetSize();
-				if (size >= 0)
+				if (size >= 0) {
 					totalSize += size;
-				else
+				}
+				else {
 					filesWithUnknownSize++;
+				}
 			}
 		}
 	}
@@ -640,8 +655,6 @@ int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, i
 		if ((*iter)->GetType() == QueueItemType::File ||
 			(*iter)->GetType() == QueueItemType::Folder)
 			queuedFiles++;
-		else if ((*iter)->GetType() == QueueItemType::FolderScan)
-			folderScanCount++;
 	}
 
 	return totalSize;
@@ -731,24 +744,6 @@ void CServerItem::SetChildPriority(CFileItem* pItem, QueuePriority oldPriority,
 	wxFAIL;
 }
 
-CFolderScanItem::CFolderScanItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const CServerPath& remotePath)
-	: CQueueItem(parent)
-	, m_queued(queued)
-	, m_localPath(localPath)
-	, m_remotePath(remotePath)
-	, m_download(download)
-{
-}
-
-bool CFolderScanItem::TryRemoveAll()
-{
-	if (!m_active)
-		return true;
-
-	m_remove = true;
-	return false;
-}
-
 // --------------
 // CQueueViewBase
 // --------------
@@ -768,24 +763,14 @@ CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
 	, m_title(title)
 {
 	m_pQueue = parent;
-	m_insertionStart = -1;
-	m_insertionCount = 0;
-	m_itemCount = 0;
-	m_allowBackgroundErase = true;
-
-	m_fileCount = 0;
-	m_folderScanCount = 0;
-	m_fileCountChanged = false;
-	m_folderScanCountChanged = false;
 
 	// Create and assign the image list for the queue
 	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
 	wxImageList* pImageList = new wxImageList(s.x, s.y);
 
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FILE"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
 
 	AssignImageList(pImageList, wxIMAGE_LIST_SMALL);
 
@@ -794,8 +779,9 @@ CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
 
 CQueueViewBase::~CQueueViewBase()
 {
-	for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-		delete *iter;
+	for (auto server : m_serverList) {
+		delete server;
+	}
 }
 
 CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item) const
@@ -917,37 +903,6 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 			}
 		}
 		break;
-	case QueueItemType::FolderScan:
-		{
-			CFolderScanItem* pFolderItem = static_cast<CFolderScanItem*>(pItem);
-			switch (column)
-			{
-			case colLocalName:
-				return _T("  ") + pFolderItem->GetLocalPath().GetPath();
-			case colDirection:
-				if (pFolderItem->Download())
-					if (pFolderItem->queued())
-						return _T("<--");
-					else
-						return _T("<<--");
-				else
-					if (pFolderItem->queued())
-						return _T("-->");
-					else
-						return _T("-->>");
-				break;
-			case colRemoteName:
-				return pFolderItem->GetRemotePath().GetPath();
-			case colTransferStatus:
-			case colErrorReason:
-				return pFolderItem->m_statusMessage;
-			case colTime:
-				return CTimeFormat::FormatDateTime(pItem->GetTime());
-			default:
-				break;
-			}
-		}
-		break;
 	case QueueItemType::Folder:
 		{
 			CFileItem* pFolderItem = static_cast<CFolderItem*>(pItem);
@@ -1025,9 +980,8 @@ int CQueueViewBase::OnGetItemImage(long item) const
 		return 0;
 	case QueueItemType::File:
 		return 1;
-	case QueueItemType::FolderScan:
 	case QueueItemType::Folder:
-			return 3;
+		return 2;
 	default:
 		return -1;
 	}
@@ -1044,20 +998,18 @@ void CQueueViewBase::UpdateSelections_ItemAdded(int added)
 #ifndef __WXMSW__
 	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
 	const int selection_count = GetSelectedItemCount();
-	if (!selection_count)
+	if (!selection_count) {
 		return;
+	}
 #endif
 
 	// Go through all items, keep record of the previous selected item
 	int item = GetNextItem(added - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
 	int prevItem = -1;
-	while (item != -1)
-	{
-		if (prevItem != -1)
-		{
-			if (prevItem + 1 != item)
-			{
+	while (item != -1) {
+		if (prevItem != -1) {
+			if (prevItem + 1 != item) {
 				// Previous selected item was not the direct predecessor
 				// That means we have to select the successor of prevItem
 				// and unselect current item
@@ -1065,8 +1017,7 @@ void CQueueViewBase::UpdateSelections_ItemAdded(int added)
 				SetItemState(item, 0, wxLIST_STATE_SELECTED);
 			}
 		}
-		else
-		{
+		else {
 			// First selected item, no predecessor yet. We have to unselect
 			SetItemState(item, 0, wxLIST_STATE_SELECTED);
 		}
@@ -1074,8 +1025,7 @@ void CQueueViewBase::UpdateSelections_ItemAdded(int added)
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	if (prevItem != -1 && prevItem < m_itemCount - 1)
-	{
+	if (prevItem != -1 && prevItem < m_itemCount - 1) {
 		// Move the very last selected item
 		SetItemState(prevItem + 1, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 	}
@@ -1085,30 +1035,38 @@ void CQueueViewBase::UpdateSelections_ItemAdded(int added)
 
 void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 {
+	wxASSERT(GetItemCount() == m_itemCount);
 #ifndef __WXMSW__
 	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
 	const int selection_count = GetSelectedItemCount();
-	if (!selection_count)
+	if (!selection_count) {
 		return;
+	}
 #endif
 
 	std::deque<int> itemsToSelect;
 
 	// Go through all selected items
 	int item = GetNextItem(added - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-
 	while (item != -1) {
 		// Select new items preceding to current one
 		while (!itemsToSelect.empty() && itemsToSelect.front() < item) {
 			SetItemState(itemsToSelect.front(), wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 			itemsToSelect.pop_front();
 		}
-		if (itemsToSelect.empty())
-			SetItemState(item, 0, wxLIST_STATE_SELECTED);
-		else if (itemsToSelect.front() == item)
+		if (itemsToSelect.front() == item) {
 			itemsToSelect.pop_front();
+		}
+		else {
+			SetItemState(item, 0, wxLIST_STATE_SELECTED);
+		}
 
-		itemsToSelect.push_back(item + count);
+		if (item + count < GetItemCount()) {
+			// On generic list controls, new items may be selected by default after
+			// increasing the item count: Internally it sometimes keeps track
+			// of only unselected items.
+			itemsToSelect.push_back(item + count);
+		}
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
@@ -1221,25 +1179,24 @@ void CQueueViewBase::CreateColumns(std::list<ColumnId> const& extraColumns)
 	LoadColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, -1, -1);
 }
 
-CServerItem* CQueueViewBase::GetServerItem(const CServer& server)
+CServerItem* CQueueViewBase::GetServerItem(ServerWithCredentials const& server)
 {
-	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
-		if ((*iter)->GetServer() == server)
+	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
+		if ((*iter)->GetServer() == server) {
 			return *iter;
+		}
 	}
 	return NULL;
 }
 
-CServerItem* CQueueViewBase::CreateServerItem(const CServer& server)
+CServerItem* CQueueViewBase::CreateServerItem(ServerWithCredentials const& server)
 {
 	CServerItem* pItem = GetServerItem(server);
 
-	if (!pItem)
-	{
+	if (!pItem) {
 		pItem = new CServerItem(server);
 		m_serverList.push_back(pItem);
-		m_itemCount++;
+		++m_itemCount;
 
 		wxASSERT(m_insertionStart == -1);
 		wxASSERT(m_insertionCount == 0);
@@ -1255,20 +1212,22 @@ void CQueueViewBase::CommitChanges()
 {
 	SaveSetItemCount(m_itemCount);
 
-	if (m_insertionStart != -1)
-	{
+	if (m_insertionStart != -1) {
 		wxASSERT(m_insertionCount != 0);
-		if (m_insertionCount == 1)
+		if (m_insertionCount == 1) {
 			UpdateSelections_ItemAdded(m_insertionStart);
-		else
+		}
+		else {
 			UpdateSelections_ItemRangeAdded(m_insertionStart, m_insertionCount);
+		}
 
 		m_insertionStart = -1;
 		m_insertionCount = 0;
 	}
 
-	if (m_fileCountChanged || m_folderScanCountChanged)
+	if (m_fileCountChanged) {
 		DisplayNumberQueuedFiles();
+	}
 }
 
 void CQueueViewBase::DisplayNumberQueuedFiles()
@@ -1280,21 +1239,14 @@ void CQueueViewBase::DisplayNumberQueuedFiles()
 
 	wxString str;
 	if (m_fileCount > 0) {
-		if (!m_folderScanCount)
-			str.Printf(m_title + _T(" (%d)"), m_fileCount);
-		else
-			str.Printf(m_title + _T(" (%d+)"), m_fileCount);
+		str.Printf(m_title + _T(" (%d)"), m_fileCount);
 	}
-	else
-	{
+	else {
 		str = m_title;
-		if (m_folderScanCount)
-			str += _T(" (0+)");
 	}
 	m_pQueue->SetPageText(m_pageIndex, str);
 
 	m_fileCountChanged = false;
-	m_folderScanCountChanged = false;
 
 	m_filecount_delay_timer.Start(200, true);
 }
@@ -1306,18 +1258,16 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 	pServerItem->AddChild(pItem);
 	m_itemCount++;
 
-	if (m_insertionStart == -1)
+	if (m_insertionStart == -1) {
+		assert(!m_insertionCount);
 		m_insertionStart = newIndex;
-	m_insertionCount++;
+	}
+	++m_insertionCount;
 
 	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		m_fileCount++;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType::FolderScan) {
-		m_folderScanCount++;
-		m_folderScanCountChanged = true;
-	}
 }
 
 bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount, bool updateSelections, bool forward)
@@ -1327,12 +1277,6 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 		m_fileCount--;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType::FolderScan) {
-		wxASSERT(m_folderScanCount > 0);
-		m_folderScanCount--;
-		m_folderScanCountChanged = true;
-
-	}
 
 	int index = 0;
 	if (updateSelections)
@@ -1379,7 +1323,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 	}
 
 	if (updateItemCount) {
-		if (m_fileCountChanged || m_folderScanCountChanged)
+		if (m_fileCountChanged)
 			DisplayNumberQueuedFiles();
 		if (oldCount > m_itemCount)
 		{
@@ -1432,13 +1376,12 @@ void CQueueViewBase::OnChar(wxKeyEvent& event)
 	m_pQueue->SetSelection(selection);
 }
 
-void CQueueViewBase::OnEndColumnDrag(wxListEvent& event)
+void CQueueViewBase::OnEndColumnDrag(wxListEvent&)
 {
-	for (unsigned int i = 0; i < m_pQueue->GetPageCount(); i++)
-	{
+	for (unsigned int i = 0; i < m_pQueue->GetPageCount(); ++i) {
 		wxWindow *page = m_pQueue->GetPage(i);
 
-		wxListCtrl* queue_page = wxDynamicCast(page, wxListCtrl);
+		wxListCtrl* queue_page = dynamic_cast<wxListCtrl*>(page);
 		if (!queue_page || queue_page == this)
 			continue;
 
@@ -1449,13 +1392,12 @@ void CQueueViewBase::OnEndColumnDrag(wxListEvent& event)
 
 void CQueueViewBase::OnTimer(wxTimerEvent& event)
 {
-	if (event.GetId() != m_filecount_delay_timer.GetId())
-	{
+	if (event.GetId() != m_filecount_delay_timer.GetId()) {
 		event.Skip();
 		return;
 	}
 
-	if (m_fileCountChanged || m_folderScanCountChanged) {
+	if (m_fileCountChanged) {
 		DisplayNumberQueuedFiles();
 	}
 }
@@ -1464,8 +1406,7 @@ void CQueueViewBase::OnKeyDown(wxKeyEvent& event)
 {
 	const int code = event.GetKeyCode();
 	const int mods = event.GetModifiers();
-	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META)))
-	{
+	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META))) {
 		for (unsigned int i = 0; i < (unsigned int)GetItemCount(); i++)
 			SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 	}
@@ -1493,7 +1434,7 @@ void CQueueViewBase::OnExport(wxCommandEvent&)
 	if (dlg.ShowModal() != wxID_OK)
 		return;
 
-	CXmlFile xml(dlg.GetPath());
+	CXmlFile xml(dlg.GetPath().ToStdWstring());
 
 	auto exportRoot = xml.CreateEmpty();
 
